@@ -3,6 +3,8 @@
 # This file is part of gunicorn released under the MIT license. 
 # See the NOTICE for more information.
 
+from __future__ import with_statement
+
 import errno
 import fcntl
 import logging
@@ -11,9 +13,11 @@ import select
 import signal
 import socket
 import sys
+import tempfile
 import time
 
 from gunicorn.worker import Worker
+from gunicorn import util
 
 class Arbiter(object):
     
@@ -31,33 +35,90 @@ class Arbiter(object):
         (getattr(signal, name), name[3:].lower()) for name in dir(signal)
         if name[:3] == "SIG" and name[3] != "_"
     )
-    
-    def __init__(self, address, num_workers, modname, debug=False):
+
+    def __init__(self, address, num_workers, modname, 
+            **kwargs):
         self.address = address
         self.num_workers = num_workers
         self.modname = modname
         self.timeout = 30
         self.reexec_pid = 0
-        self.pid = os.getpid()
-        self.debug = debug
+        self.debug = kwargs.get("debug", False)
         self.log = logging.getLogger(__name__)
+        self.opts = kwargs
+        self._pidfile = None
+        
+        
+    def start(self):
+        self.pid = os.getpid()
         self.init_signals()
         self.listen(self.address)
+        self.pidfile = self.opts.get("pidfile")
         self.log.info("Booted Arbiter: %s" % os.getpid())
         
-                    
+    def _del_pidfile(self):
+        self._pidfile = None
+        
+    def _get_pidfile(self):
+        return self._pidfile
+        
+    def _set_pidfile(self, path):
+        if not path:
+            return
+            
+        pid = self.valid_pidfile(path)
+        if pid:
+            if self.pidfile and path == self.pidfile and pid == os.getpid():
+                return path
+            raise RuntimeError("Already running on PID %s " \
+                        "(or pid file '%s' is stale)" % (os.getpid(), path))
+        if self.pidfile:    
+            self.unlink_pidfile(self.pidfile)
+
+        # write pidfile
+        fd, fname = tempfile.mkstemp()
+        os.write(fd, "%s\n" % self.pid)
+        os.rename(fname, path)
+        os.close(fd)
+        self._pidfile = path
+    pidfile = property(_get_pidfile, _set_pidfile, _del_pidfile)
+    
+         
+    def unlink_pidfile(self, path):
+        try:
+            with open(path, "r") as f:
+                if int(f.read() or 0) == self.pid:
+                    os.unlink(f)
+        except:
+            pass
+        
+    def valid_pidfile(self, path):
+        try:
+            with open(path, "r") as f:
+                wpid = int(f.read() or 0)
+
+                if wpid <= 0: return None
+     
+                try:
+                    os.kill(wpid, 0)
+                    return wpid
+                except OSError, e:
+                    if e[0] == errno.ESRCH:
+                        return
+                    raise
+        except IOError, e:
+            if e[0] == errno.ENOENT:
+                return
+            raise
+    
     def init_signals(self):
         if self.PIPE:
             map(lambda p: p.close(), self.PIPE)
         self.PIPE = pair = os.pipe()
-        map(self.set_non_blocking, pair)
-        map(lambda p: fcntl.fcntl(p, fcntl.F_SETFD, fcntl.FD_CLOEXEC), pair)
+        map(util.set_non_blocking, pair)
+        map(util.close_on_exec, pair)
         map(lambda s: signal.signal(s, self.signal), self.SIGNALS)
         signal.signal(signal.SIGCHLD, self.handle_chld)
-    
-    def set_non_blocking(self, fd):
-        flags = fcntl.fcntl(fd, fcntl.F_GETFL) | os.O_NONBLOCK
-        fcntl.fcntl(fd, fcntl.F_SETFL, flags)
 
     def signal(self, sig, frame):
         if len(self.SIG_QUEUE) < 5:
@@ -65,7 +126,6 @@ class Arbiter(object):
             self.wakeup()
         else:
             self.log.warn("Ignoring rapid signaling: %s" % sig)
-        
 
     def listen(self, addr):
         if 'GUNICORN_FD' in os.environ:
@@ -80,7 +140,7 @@ class Arbiter(object):
                     self.log.error("should be a non GUNICORN environnement")
                 else:
                     raise
-        
+
         for i in range(5):
             try:
                 sock = self.init_socket(addr)
@@ -92,6 +152,8 @@ class Arbiter(object):
                 if i < 5:
                     self.log.error("Retrying in 1 second.")
                 time.sleep(1)
+        if self.LISTENER: 
+            self.log.info("Listen on %s:%s" % self.LISTENER.getsockname())
                 
     def init_socket_fromfd(self, fd, address):
         sock = socket.fromfd(fd, socket.AF_INET, socket.SOCK_STREAM)
@@ -115,6 +177,7 @@ class Arbiter(object):
             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NOPUSH, 1)
 
     def run(self):
+        self.start()
         self.manage_workers()
         while True:
             try:
@@ -141,18 +204,22 @@ class Arbiter(object):
             except StopIteration:
                 break
             except KeyboardInterrupt:
-                self.stop(False)
+                self.stop()
                 sys.exit(-1)
             except Exception:
-                self.log.exception("Unhandled exception in main loop.")
+                self.log.info("Unhandled exception in main loop.")
                 self.stop(False)
                 sys.exit(-1)
-                
-        self.log.info("Master is shutting down.")
+
         self.stop()
+        self.log.info("Master is shutting down.")
+        if self.pidfile:
+            self.unlink_pidfile(self.pidfile)
+        sys.exit(0)
         
     def handle_chld(self, sig, frame):
         self.wakeup()
+        self.reap_workers()
         
     def handle_hup(self):
         self.log.info("Master hang up.")
@@ -220,12 +287,12 @@ class Arbiter(object):
         if not graceful:
             sig = signal.SIGTERM
         limit = time.time() + self.timeout
-        while len(self.WORKERS) and time.time() < limit:
+        while self.WORKERS or time.time() > limit:
             self.kill_workers(sig)
             time.sleep(0.1)
             self.reap_workers()
         self.kill_workers(signal.SIGKILL)
-        
+
     def reexec(self):
         self.reexec_pid = os.fork()
         if self.reexec_pid == 0:
@@ -237,7 +304,7 @@ class Arbiter(object):
             diff = time.time() - os.fstat(worker.tmp.fileno()).st_ctime
             if diff <= self.timeout:
                 continue
-            self.log.error("worker %s PID %s timeout killing." % (str(worker.id), pid))
+            self.log.error("%s (pid:%s) timed out." % (worker, pid))
             self.kill_worker(pid, signal.SIGKILL)
     
     def reap_workers(self):
@@ -271,12 +338,12 @@ class Arbiter(object):
                 continue
 
             worker = Worker(i, self.pid, self.LISTENER, self.modname,
-                        self.timeout, self.debug)
+                        self.timeout/2, self.debug)
             pid = os.fork()
             if pid != 0:
                 self.WORKERS[pid] = worker
                 continue
-            
+
             # Process Child
             worker_pid = os.getpid()
             try:
@@ -284,13 +351,11 @@ class Arbiter(object):
                 worker.run()
                 sys.exit(0)
             except SystemExit:
-                
                 raise
             except:
                 self.log.exception("Exception in worker process.")
                 sys.exit(-1)
             finally:
-                worker.tmp.close()
                 self.log.info("Worker %s exiting." % worker_pid)
 
     def kill_workers(self, sig):
@@ -298,16 +363,13 @@ class Arbiter(object):
             self.kill_worker(pid, sig)
         
     def kill_worker(self, pid, sig):
-        worker = self.WORKERS.pop(pid)
         try:
             os.kill(pid, sig)
-            kpid, stat = os.waitpid(pid, os.WNOHANG)
-            if kpid:
-                self.log.warning("Problem killing process: %s" % pid)
         except OSError, e:
             if e.errno == errno.ESRCH:
+                worker = self.WORKERS.pop(pid)
                 try:
                     worker.tmp.close()
                 except:
                     pass
-
+            raise
